@@ -1,7 +1,7 @@
 #!/usr/bin/env python
 # -*- coding: utf-8 -*-
 #
-# Copyright (c) 2012-2018 Snowflake Computing Inc. All right reserved.
+# Copyright (c) 2012-2019 Snowflake Computing Inc. All right reserved.
 #
 """
 Various constants
@@ -99,6 +99,10 @@ def is_timestamp_type_name(type_name):
     return type_name in __timestamp_type_names
 
 
+def is_date_type_name(type_name):
+    return type_name == u'DATE'
+
+
 # Log format
 LOG_FORMAT = (u'%(asctime)s - %(filename)s:%(lineno)d - '
               u'%(funcName)s() - %(levelname)s - %(message)s')
@@ -131,12 +135,15 @@ FileHeader = namedtuple(
 PARAMETER_AUTOCOMMIT = u'AUTOCOMMIT'
 PARAMETER_CLIENT_SESSION_KEEP_ALIVE_HEARTBEAT_FREQUENCY = u'CLIENT_SESSION_KEEP_ALIVE_HEARTBEAT_FREQUENCY'
 PARAMETER_CLIENT_SESSION_KEEP_ALIVE = u'CLIENT_SESSION_KEEP_ALIVE'
+PARAMETER_CLIENT_PREFETCH_THREADS = u'CLIENT_PREFETCH_THREADS'
 PARAMETER_CLIENT_TELEMETRY_ENABLED = u'CLIENT_TELEMETRY_ENABLED'
+PARAMETER_CLIENT_TELEMETRY_OOB_ENABLED = u'CLIENT_OUT_OF_BAND_TELEMETRY_ENABLED'
 PARAMETER_CLIENT_STORE_TEMPORARY_CREDENTIAL = u'CLIENT_STORE_TEMPORARY_CREDENTIAL'
 PARAMETER_CLIENT_USE_SECURE_STORAGE_FOR_TEMPORARY_CREDENTIAL = \
     u'CLIENT_USE_SECURE_STORAGE_FOR_TEMPORARY_CREDENTAIL'
 PARAMETER_TIMEZONE = u'TIMEZONE'
 PARAMETER_SERVICE_NAME = u'SERVICE_NAME'
+PARAMETER_CLIENT_VALIDATE_DEFAULT_PARAMETERS = u'CLIENT_VALIDATE_DEFAULT_PARAMETERS'
 
 HTTP_HEADER_CONTENT_TYPE = u'Content-Type'
 HTTP_HEADER_ACCEPT = u"accept"
@@ -144,3 +151,12 @@ HTTP_HEADER_USER_AGENT = u"User-Agent"
 HTTP_HEADER_SERVICE_NAME = u'X-Snowflake-Service'
 
 HTTP_HEADER_VALUE_OCTET_STREAM = u'application/octet-stream'
+
+
+class OCSPMode(Enum):
+    """
+    OCSP Mode
+    """
+    FAIL_CLOSED = u'FAIL_CLOSED'
+    FAIL_OPEN = u'FAIL_OPEN'
+    INSECURE = u'INSECURE'
