@@ -1,7 +1,7 @@
 #!/usr/bin/env python
 # -*- coding: utf-8 -*-
 #
-# Copyright (c) 2012-2018 Snowflake Computing Inc. All right reserved.
+# Copyright (c) 2012-2019 Snowflake Computing Inc. All right reserved.
 #
 import logging
 import re
@@ -10,29 +10,43 @@ import sys
 import uuid
 from logging import getLogger
 from threading import (Timer, Lock)
-
 from six import u
 
-from .chunk_downloader import (DEFAULT_CLIENT_RESULT_PREFETCH_SLOTS,
-                               DEFAULT_CLIENT_RESULT_PREFETCH_THREADS)
 from .compat import (BASE_EXCEPTION_CLASS)
 from .constants import (
     FIELD_NAME_TO_ID,
-    FIELD_ID_TO_NAME,
 )
 from .errorcode import (ER_UNSUPPORTED_METHOD,
                         ER_CURSOR_IS_CLOSED,
                         ER_FAILED_TO_REWRITE_MULTI_ROW_INSERT,
                         ER_NOT_POSITIVE_SIZE,
-                        ER_FAILED_TO_CONVERT_ROW_TO_PYTHON_TYPE,
-                        ER_INVALID_VALUE)
+                        ER_INVALID_VALUE,
+                        ER_NO_PYARROW,
+                        ER_NO_ARROW_RESULT,
+                        ER_NO_PYARROW_SNOWSQL)
 from .errors import (Error, ProgrammingError, NotSupportedError,
                      DatabaseError, InterfaceError)
 from .file_transfer_agent import (SnowflakeFileTransferAgent)
+from .json_result import JsonResult, DictJsonResult
 from .sqlstate import (SQLSTATE_FEATURE_NOT_SUPPORTED)
 from .telemetry import (TelemetryData, TelemetryField)
 from .time_util import get_time_millis
 
+logger = getLogger(__name__)
+
+try:
+    import pyarrow
+except ImportError:
+    logger.debug("Failed to import pyarrow. No Apache Arrow result set format can be used.")
+    pyarrow = None
+
+try:
+    from .arrow_result import ArrowResult
+    CAN_USE_ARROW_RESULT = True
+except ImportError:
+    logger.debug("Failed to import ArrowResult. No Apache Arrow result set format can be used.")
+    CAN_USE_ARROW_RESULT = False
+
 STATEMENT_TYPE_ID_DML = 0x3000
 STATEMENT_TYPE_ID_INSERT = STATEMENT_TYPE_ID_DML + 0x100
 STATEMENT_TYPE_ID_UPDATE = STATEMENT_TYPE_ID_DML + 0x200
@@ -49,7 +63,7 @@ STATEMENT_TYPE_ID_DML_SET = frozenset(
 DESC_TABLE_RE = re.compile(u(r'desc(?:ribe)?\s+([\w_]+)\s*;?\s*$'),
                            flags=re.IGNORECASE)
 
-logger = getLogger(__name__)
+LOG_MAX_QUERY_LENGTH = 80
 
 
 class SnowflakeCursor(object):
@@ -67,9 +81,7 @@ class SnowflakeCursor(object):
         u(r'alter\s+session\s+set\s+(.*)=\'?([^\']+)\'?\s*;'),
         flags=re.IGNORECASE | re.MULTILINE | re.DOTALL)
 
-    LOG_MAX_QUERY_LENGTH = 80
-
-    def __init__(self, connection):
+    def __init__(self, connection, json_result_class=JsonResult):
         self._connection = connection
 
         self._errorhandler = Error.default_errorhandler
@@ -92,24 +104,23 @@ class SnowflakeCursor(object):
         self._time_output_format = None
         self._timezone = None
         self._binary_output_format = None
+        self._result = None
+        self._json_result_class = json_result_class
 
-        self._client_result_prefetch_slots = \
-            DEFAULT_CLIENT_RESULT_PREFETCH_SLOTS
-        self._client_result_prefetch_threads = \
-            DEFAULT_CLIENT_RESULT_PREFETCH_THREADS
         self._arraysize = 1  # PEP-0249: defaults to 1
 
         self._lock_canceling = Lock()
 
         self._first_chunk_time = None
 
+        self._log_max_query_length = connection.log_max_query_length
+
         self.reset()
 
     def __del__(self):
         try:
             self.close()
         except BASE_EXCEPTION_CLASS as e:
-            logger = getLogger(__name__)
             if logger.getEffectiveLevel() <= logging.INFO:
                 logger.info(e)
 
@@ -141,7 +152,7 @@ class SnowflakeCursor(object):
         The current 0-based index of the cursor in the result set or None if
         the index cannot be determined.
         """
-        return self._total_row_index if self._total_row_index >= 0 else None
+        return self._result.total_row_index if self._result.total_row_index >= 0 else None
 
     @property
     def sfqid(self):
@@ -309,9 +320,7 @@ class SnowflakeCursor(object):
 
         if logger.getEffectiveLevel() <= logging.DEBUG:
             logger.debug(
-                u'running query [%s]',
-                u' '.join(line.strip() for line in query.split(u'\n')),
-            )
+                u'running query [%s]', self._format_query_for_log(query))
         if _is_put_get is not None:
             # if told the query is PUT or GET, use the information
             self._is_file_transfer = _is_put_get
@@ -381,6 +390,9 @@ class SnowflakeCursor(object):
                 logger.debug(
                     u'Failed to reset SIGINT handler. Not in main '
                     u'thread. Ignored...')
+            except Exception:
+                self.connection.incident.report_incident()
+                raise
             if self._timebomb is not None:
                 self._timebomb.cancel()
                 logger.debug(u'cancelled timebomb in finally')
@@ -403,10 +415,6 @@ class SnowflakeCursor(object):
                     self._timezone = kv[u'value']
                 if u'BINARY_OUTPUT_FORMAT' in kv[u'name']:
                     self._binary_output_format = kv[u'value']
-                if u'CLIENT_RESULT_PREFETCH_THREADS' in kv[u'name']:
-                    self._client_result_prefetch_threads = kv[u'value']
-                if u'CLIENT_RESULT_PREFETCH_SLOTS' in kv[u'name']:
-                    self._client_result_prefetch_slots = kv[u'value']
             self._connection._set_parameters(
                 ret, self._connection._session_parameters)
 
@@ -416,15 +424,19 @@ class SnowflakeCursor(object):
     def execute(self, command, params=None, timeout=None,
                 _do_reset=True,
                 _put_callback=None,
+                _put_azure_callback=None,
                 _put_callback_output_stream=sys.stdout,
                 _get_callback=None,
+                _get_azure_callback=None,
                 _get_callback_output_stream=sys.stdout,
+                _show_progress_bar=True,
                 _statement_params=None,
                 _is_internal=False,
                 _no_results=False,
                 _use_ijson=False,
                 _is_put_get=None,
-                _raise_put_get_error=False):
+                _raise_put_get_error=False,
+                _force_put_overwrite=False):
         u"""
         Executes a command/query
         """
@@ -443,24 +455,36 @@ class SnowflakeCursor(object):
             logger.warning(u'execute: no query is given to execute')
             return
 
-        if self._connection.is_pyformat:
-            # pyformat/format paramstyle
-            # client side binding
-            processed_params = self._connection._process_params(params, self)
-            logger.debug(u'binding: %s with input=%s, processed=%s',
-                         command,
-                         params, processed_params)
-            if len(processed_params) > 0:
-                query = command % processed_params
+        try:
+            if self._connection.is_pyformat:
+                # pyformat/format paramstyle
+                # client side binding
+                processed_params = self._connection._process_params(params, self)
+                if logger.getEffectiveLevel() <= logging.DEBUG:
+                    logger.debug(u'binding: [%s] with input=[%s], processed=[%s]',
+                                 self._format_query_for_log(command),
+                                 params, processed_params)
+                if len(processed_params) > 0:
+                    query = command % processed_params
+                else:
+                    query = command
+                processed_params = None  # reset to None
             else:
+                # qmark and numeric paramstyle
+                # server side binding
                 query = command
-            processed_params = None  # reset to None
-        else:
-            # qmark and numeric paramstyle
-            # server side binding
-            query = command
-            processed_params = self._connection._process_params_qmarks(
-                params, self)
+                processed_params = self._connection._process_params_qmarks(
+                    params, self)
+        # Skip reporting Key, Value and Type errors
+        except KeyError:
+            raise
+        except ValueError:
+            raise
+        except TypeError:
+            raise
+        except Exception:
+            self.connection.incident.report_incident()
+            raise
 
         m = DESC_TABLE_RE.match(query)
         if m:
@@ -512,10 +536,14 @@ class SnowflakeCursor(object):
                 sf_file_transfer_agent = SnowflakeFileTransferAgent(
                     self, query, ret,
                     put_callback=_put_callback,
+                    put_azure_callback=_put_azure_callback,
                     put_callback_output_stream=_put_callback_output_stream,
                     get_callback=_get_callback,
+                    get_azure_callback=_get_azure_callback,
                     get_callback_output_stream=_get_callback_output_stream,
-                    raise_put_get_error=_raise_put_get_error)
+                    show_progress_bar=_show_progress_bar,
+                    raise_put_get_error=_raise_put_get_error,
+                    force_put_overwrite=_force_put_overwrite)
                 sf_file_transfer_agent.execute()
                 data = sf_file_transfer_agent.result()
                 self._total_rowcount = len(data[u'rowset']) if \
@@ -532,7 +560,7 @@ class SnowflakeCursor(object):
                     u'total'] if u'data' in ret and u'total' in ret[
                     u'data'] else -1
                 return data
-            self.chunk_info(data, use_ijson=_use_ijson)
+            self._init_result_and_meta(data, _use_ijson)
         else:
             self._total_rowcount = ret[u'data'][
                 u'total'] if u'data' in ret and u'total' in ret[u'data'] else -1
@@ -553,27 +581,24 @@ class SnowflakeCursor(object):
         return self
 
     def _format_query_for_log(self, query):
-        ret = u' '.join(line.strip() for line in query.split(u'\n'))
-        return (ret if len(ret) < SnowflakeCursor.LOG_MAX_QUERY_LENGTH
-                else ret[0:SnowflakeCursor.LOG_MAX_QUERY_LENGTH] + '...')
+        return self._connection._format_query_for_log(query)
 
     def _is_dml(self, data):
         return u'statementTypeId' in data \
                and int(data[u'statementTypeId']) in \
                STATEMENT_TYPE_ID_DML_SET
 
-    def chunk_info(self, data, use_ijson=False):
+    def _init_result_and_meta(self, data, use_ijson=False):
         is_dml = self._is_dml(data)
+        self._query_result_format = data.get(u'queryResultFormat', u'json')
 
         if self._total_rowcount == -1 and not is_dml and data.get(u'total') \
                 is not None:
             self._total_rowcount = data['total']
 
         self._description = []
-        self._column_idx_to_name = {}
-        self._column_converter = []
-        for idx, column in enumerate(data[u'rowtype']):
-            self._column_idx_to_name[idx] = column[u'name']
+
+        for column in data[u'rowtype']:
             type_value = FIELD_NAME_TO_ID[column[u'type'].upper()]
             self._description.append((column[u'name'],
                                       type_value,
@@ -582,40 +607,12 @@ class SnowflakeCursor(object):
                                       column[u'precision'],
                                       column[u'scale'],
                                       column[u'nullable']))
-            self._column_converter.append(
-                self._connection.converter.to_python_method(
-                    column[u'type'].upper(), column))
 
-        self._total_row_index = -1  # last fetched number of rows
-
-        self._chunk_index = 0
-        self._chunk_count = 0
-        self._current_chunk_row = iter(data.get(u'rowset'))
-        self._current_chunk_row_count = len(data.get(u'rowset'))
-
-        if u'chunks' in data:
-            chunks = data[u'chunks']
-            self._chunk_count = len(chunks)
-            logger.debug(u'chunk size=%s', self._chunk_count)
-            # prepare the downloader for further fetch
-            qrmk = data[u'qrmk'] if u'qrmk' in data else None
-            chunk_headers = None
-            if u'chunkHeaders' in data:
-                chunk_headers = {}
-                for header_key, header_value in data[
-                    u'chunkHeaders'].items():
-                    chunk_headers[header_key] = header_value
-                    logger.debug(
-                        u'added chunk header: key=%s, value=%s',
-                        header_key,
-                        header_value)
-
-            logger.debug(u'qrmk=%s', qrmk)
-            self._chunk_downloader = self._connection._chunk_downloader_class(
-                chunks, self._connection, self, qrmk, chunk_headers,
-                prefetch_slots=self._client_result_prefetch_slots,
-                prefetch_threads=self._client_result_prefetch_threads,
-                use_ijson=use_ijson)
+        if self._query_result_format == 'arrow':
+            self.check_pyarrow_resultset()
+            self._result = ArrowResult(data, self)
+        else:
+            self._result = self._json_result_class(data, self, use_ijson)
 
         if is_dml:
             updated_rows = 0
@@ -631,8 +628,38 @@ class SnowflakeCursor(object):
             else:
                 self._total_rowcount += updated_rows
 
+    def check_pyarrow_resultset(self):
+        global CAN_USE_ARROW_RESULT
+        global pyarrow
+
+        if not CAN_USE_ARROW_RESULT:
+            if self._connection.application == 'SnowSQL':
+                msg = (
+                    "Currently SnowSQL doesn't support the result set in Apache Arrow format."
+                )
+                errno = ER_NO_PYARROW_SNOWSQL
+            elif pyarrow is None:
+                msg = (
+                    "pyarrow package is missing. Install using pip if the platform is supported."
+                )
+                errno = ER_NO_PYARROW
+            else:
+                msg = (
+                    "The result set in Apache Arrow format is not supported for the platform."
+                )
+                errno = ER_NO_ARROW_RESULT
+
+            Error.errorhandler_wrapper(
+                self.connection, self,
+                ProgrammingError,
+                {
+                    u'msg': msg,
+                    u'errno': errno,
+                }
+            )
+
     def query_result(self, qid, _use_ijson=False):
-        url = ('/queries/{qid}/result').format(qid=qid)
+        url = '/queries/{qid}/result'.format(qid=qid)
         ret = self._connection.rest.request(url=url, method='get')
         self._sfqid = ret[u'data'][
             u'queryId'] if u'data' in ret and u'queryId' in ret[
@@ -644,7 +671,7 @@ class SnowflakeCursor(object):
 
         if ret.get(u'success'):
             data = ret.get(u'data')
-            self.chunk_info(data, use_ijson=_use_ijson)
+            self._init_result_and_meta(data, _use_ijson)
         else:
             logger.info(u'failed')
             logger.debug(ret)
@@ -663,6 +690,25 @@ class SnowflakeCursor(object):
                                        errvalue)
         return self
 
+    def fetch_pandas_batches(self, **kwargs):
+        u"""
+        Fetch a single Arrow Table
+        @param kwargs: will be passed to pyarrow.Table.to_pandas() method
+        """
+        if self._query_result_format != 'arrow':  # TODO: or pandas isn't imported
+            raise NotSupportedError
+        for df in self._result._fetch_pandas_batches(**kwargs):
+            yield df
+
+    def fetch_pandas_all(self, **kwargs):
+        u"""
+        Fetch Pandas dataframes in batch, where 'batch' refers to Snowflake Chunk
+        @param kwargs: will be passed to pyarrow.Table.to_pandas() method
+        """
+        if self._query_result_format != 'arrow':
+            raise NotSupportedError
+        return self._result._fetch_pandas_all(**kwargs)
+
     def abort_query(self, qid):
         url = '/queries/{qid}/abort-request'.format(qid=qid)
         ret = self._connection.rest.request(url=url, method='post')
@@ -742,53 +788,10 @@ class SnowflakeCursor(object):
         """
         Fetch one row
         """
-        is_done = False
         try:
-            row = None
-            self._total_row_index += 1
-            try:
-                row = next(self._current_chunk_row)
-            except StopIteration:
-                if self._chunk_index < self._chunk_count:
-                    logger.debug(
-                        u"chunk index: %s, chunk_count: %s",
-                        self._chunk_index, self._chunk_count)
-                    next_chunk = self._chunk_downloader.next_chunk()
-                    self._current_chunk_row_count = next_chunk.row_count
-                    self._current_chunk_row = next_chunk.result_data
-                    self._chunk_index += 1
-                    try:
-                        row = next(self._current_chunk_row)
-                    except StopIteration:
-                        is_done = True
-                        raise IndexError
-                else:
-                    if self._chunk_count > 0 and \
-                            self._chunk_downloader is not None:
-                        self._chunk_downloader.terminate()
-                        self._log_telemetry_job_data(
-                            TelemetryField.TIME_DOWNLOADING_CHUNKS,
-                            self._chunk_downloader._total_millis_downloading_chunks)
-                        self._log_telemetry_job_data(
-                            TelemetryField.TIME_PARSING_CHUNKS,
-                            self._chunk_downloader._total_millis_parsing_chunks)
-                    self._chunk_downloader = None
-                    self._chunk_count = 0
-                    self._current_chunk_row = iter(())
-                    is_done = True
-
-            return self._row_to_python(row) if row is not None else None
-
-        except IndexError:
-            # returns None if the iteration is completed so that iter() stops
+            return next(self._result)
+        except StopIteration:
             return None
-        finally:
-            if is_done and self._first_chunk_time:
-                logger.info("fetching data done")
-                time_consume_last_result = get_time_millis() - self._first_chunk_time
-                self._log_telemetry_job_data(
-                    TelemetryField.TIME_CONSUME_LAST_RESULT,
-                    time_consume_last_result)
 
     def fetchmany(self, size=None):
         u"""
@@ -862,23 +865,14 @@ class SnowflakeCursor(object):
         Reset the result set
         """
         self._total_rowcount = -1  # reset the rowcount
-        self._total_row_index = -1  # last fetched number of rows
-        self._current_chunk_row_count = 0
-        self._current_chunk_row = iter(())
-        self._chunk_index = 0
-
-        if hasattr(self, u'_chunk_count') and self._chunk_count > 0 and \
-                self._chunk_downloader is not None:
-            self._chunk_downloader.terminate()
-
-        self._chunk_count = 0
-        self._chunk_downloader = None
+        if self._result is not None:
+            self._result._reset()
 
     def __iter__(self):
         u"""
         Iteration over the result set
         """
-        return iter(self.fetchone, None)
+        return iter(self._result)
 
     def __cancel_query(self, query):
         if self._sequence_counter >= 0 and not self.is_closed():
@@ -887,37 +881,6 @@ class SnowflakeCursor(object):
             with self._lock_canceling:
                 self._connection._cancel_query(query, self._request_id)
 
-    def _row_to_python(self, row):
-        """
-        Converts data in row if required.
-
-        NOTE: surprisingly using idx+1 is faster than enumerate here. Also
-        removing generator improved performance even better.
-        """
-        idx = 0
-        for col in row:
-            conv = self._column_converter[idx]
-            try:
-                row[idx] = col if conv is None or col is None else conv(col)
-            except Exception as e:
-                col_desc = self._description[idx]
-                msg = u'Failed to convert: ' \
-                      u'field {name}: {type}::{value}, Error: ' \
-                      u'{error}'.format(
-                    name=col_desc[0],
-                    type=FIELD_ID_TO_NAME[col_desc[1]],
-                    value=col,
-                    error=e
-                )
-                logger.exception(msg)
-                Error.errorhandler_wrapper(
-                    self.connection, self, InterfaceError, {
-                        u'msg': msg,
-                        u'errno': ER_FAILED_TO_CONVERT_ROW_TO_PYTHON_TYPE,
-                    })
-            idx += 1
-        return tuple(row)
-
     def _log_telemetry_job_data(self, telemetry_field, value):
         u"""
         Builds an instance of TelemetryData with the given field and logs it
@@ -954,33 +917,4 @@ class DictCursor(SnowflakeCursor):
     """
 
     def __init__(self, connection):
-        SnowflakeCursor.__init__(self, connection)
-
-    def _row_to_python(self, row):
-        # see the base class
-        res = {}
-        idx = 0
-        for col in row:
-            col_name = self._column_idx_to_name[idx]
-            conv = self._column_converter[idx]
-            try:
-                res[col_name] = col if conv is None or col is None else conv(
-                    col)
-            except Exception as e:
-                col_desc = self._description[idx]
-                msg = u'Failed to convert: ' \
-                      u'field {name}: {type}::{value}, Error: ' \
-                      u'{error}'.format(
-                    name=col_desc[0],
-                    type=FIELD_ID_TO_NAME[col_desc[1]],
-                    value=col,
-                    error=e
-                )
-                logger.exception(msg)
-                Error.errorhandler_wrapper(
-                    self.connection, self, InterfaceError, {
-                        u'msg': msg,
-                        u'errno': ER_FAILED_TO_CONVERT_ROW_TO_PYTHON_TYPE,
-                    })
-            idx += 1
-        return res
+        SnowflakeCursor.__init__(self, connection, DictJsonResult)
