@@ -1,7 +1,7 @@
 #!/usr/bin/env python
 # -*- coding: utf-8 -*-
 #
-# Copyright (c) 2012-2018 Snowflake Computing Inc. All right reserved.
+# Copyright (c) 2012-2019 Snowflake Computing Inc. All right reserved.
 #
 u"""This module contains Snowflake error codes"""
 
@@ -67,6 +67,11 @@ ER_OPENSSL_IS_NOT_ACCESSIBLE = 254006
 ER_INVALID_OCSP_RESPONSE = 254007
 ER_CA_CERTIFICATE_NOT_FOUND = 254008
 ER_SERVER_CERTIFICATE_UNKNOWN = 254009
+ER_INVALID_OCSP_RESPONSE_CODE = 254010
+ER_INVALID_SSD = 254011
 
 # converter
 ER_NOT_SUPPORT_DATA_TYPE = 255001
+ER_NO_PYARROW = 255002
+ER_NO_ARROW_RESULT = 255003
+ER_NO_PYARROW_SNOWSQL = 255004
