@@ -1,27 +1,13 @@
 #!/usr/bin/env python
 # -*- coding: utf-8 -*-
 #
-# Copyright (c) 2012-2018 Snowflake Computing Inc. All right reserved.
+# Copyright (c) 2012-2019 Snowflake Computing Inc. All right reserved.
 #
-import logging
 import os
 import sys
-from logging import getLogger
 from os import path
 
-from snowflake.connector.ocsp_asn1crypto import (
-    read_cert_bundle,
-    _create_pair_issuer_subject)
-
-for logger_name in ['__main__', 'snowflake']:
-    logger = logging.getLogger(logger_name)
-    logger.setLevel(logging.INFO)
-    ch = logging.StreamHandler()
-    ch.setLevel(logging.INFO)
-    ch.setFormatter(logging.Formatter('%(message)s'))
-    logger.addHandler(ch)
-
-logger = getLogger(__name__)
+from snowflake.connector.ocsp_asn1crypto import SnowflakeOCSPAsn1Crypto
 
 
 def main():
@@ -55,14 +41,14 @@ Usage: {0}  <input file/dir>
 
 
 def extract_certificate_file(input_filename):
+    ocsp = SnowflakeOCSPAsn1Crypto()
     cert_map = {}
-    read_cert_bundle(input_filename, cert_map)
-    cert_data = _create_pair_issuer_subject(cert_map)
+    ocsp.read_cert_bundle(input_filename, cert_map)
 
-    for issuer, subject in cert_data:
+    for cert in cert_map.values():
         print("serial #: {}, name: {}".format(
-            subject.serial_number,
-            subject.subject.native))
+            cert.serial_number,
+            cert.subject.native))
 
 
 if __name__ == '__main__':
